@@ -518,9 +518,19 @@ my_strusage (int level)
     case 11: p = "gpg-error"; break;
     case 12: p = PACKAGE_NAME; break;
     case 13: p = PACKAGE_VERSION; break;
-    case 14: p = "Copyright (C) 2019 g10 Code GmbH"; break;
+    case 14: p = GPGRT_STD_COPYRIGHT_LINE; break;
     case 19: p = _("Please report bugs to <https://bugs.gnupg.org>.\n"); break;
 
+#ifdef HAVE_W32_SYSTEM
+    case 31:
+#if defined(_WIN64)
+      p = "(Windows 64 bit)";
+#else
+      p = "(Windows 32 bit)";
+#endif
+      break;
+#endif
+
     case 1:
     case 40:
       p = ("Usage: gpg-error [options] error-numbers");
@@ -548,6 +558,8 @@ main (int argc, char *argv[])
          CMD_LIST,
          CMD_DEFINES,
          CMD_LOCALE,
+         CMD_GETREG,
+         CMD_FOPEN,
          OPT_DESC
   };
   static gpgrt_opt_t opts[] = {
@@ -559,11 +571,17 @@ main (int argc, char *argv[])
                 "Print all error codes as #define lines"),
 #if HAVE_W32_SYSTEM
     ARGPARSE_c (CMD_LOCALE, "locale",
-                "Return the locale used for gettext"),
+                "Print the locale used for gettext"),
+    ARGPARSE_c (CMD_GETREG, "getreg",
+                "Print string from the Registry"),
 #else
     ARGPARSE_c (CMD_LOCALE, "locale",
                 "@"),
+    ARGPARSE_c (CMD_GETREG, "getreg",
+                "@"),
 #endif
+    ARGPARSE_c (CMD_FOPEN, "fopen",
+                "Open the given file"),
     ARGPARSE_s_n (OPT_DESC, "desc",
                   "Print with error description"),
     ARGPARSE_end()
@@ -574,6 +592,8 @@ main (int argc, char *argv[])
   int libversion = 0;
   int listmode = 0;
   int localemode = 0;
+  int getregmode = 0;
+  int cmdfopen = 0;
   int desc = 0;
   const char *s, *s2;
   const char *source_sym;
@@ -583,7 +603,8 @@ main (int argc, char *argv[])
   gpgrt_init ();
   i18n_init ();
   gpgrt_set_strusage (my_strusage);
-  gpgrt_log_set_prefix (gpgrt_strusage (11), GPGRT_LOG_WITH_PREFIX);
+  gpgrt_log_set_prefix (gpgrt_strusage (11),
+                        (GPGRT_LOG_WITH_PREFIX| GPGRT_LOG_NO_REGISTRY));
 
 
   while (gpgrt_argparse (NULL, &pargs, opts))
@@ -594,6 +615,8 @@ main (int argc, char *argv[])
         case CMD_LIST:       listmode = 1; break;
         case CMD_DEFINES:    listmode = 2; break;
         case CMD_LOCALE:     localemode = 1; break;
+        case CMD_GETREG:     getregmode = 1; break;
+        case CMD_FOPEN:      cmdfopen = 1; break;
         case OPT_DESC:       desc = 1; break;
         default: pargs.err = ARGPARSE_PRINT_WARNING; break;
         }
@@ -610,6 +633,11 @@ main (int argc, char *argv[])
       if (argc > 1)
         gpgrt_usage (1);
     }
+  else if (getregmode|| cmdfopen)
+    {
+      if (argc != 1)
+        gpgrt_usage (1);
+    }
   else if ((argc && listmode) || (!argc && !listmode))
     gpgrt_usage (1);
 
@@ -656,6 +684,28 @@ main (int argc, char *argv[])
       printf ("%s\n", gettext_localename ());
 #else
       log_info ("this command is only useful on Windows\n");
+#endif
+    }
+  else if (cmdfopen)
+    {
+      gpgrt_stream_t fp;
+
+      fp = gpgrt_fopen (*argv, "r,sysopen");
+      if (!fp)
+        log_error ("error opening '%s': %s\n",
+                   *argv, gpg_strerror (gpg_error_from_syserror ()));
+      else
+        {
+          printf ("success opening '%s'\n", *argv);
+          gpgrt_fclose (fp);
+        }
+    }
+  else if (getregmode)
+    {
+#if HAVE_W32_SYSTEM
+      printf ("%s\n", gpgrt_w32_reg_get_string (*argv));
+#else
+      log_info ("this command is only useful on Windows\n");
 #endif
     }
   else if (listmode == 1)
