@@ -137,14 +137,29 @@ _gpgrt_setenv (const char *name, const char *value, int overwrite)
   {
     int exists;
     char tmpbuf[10];
-    char *buf;
 
     if (!value && overwrite)
       {
         if (!SetEnvironmentVariable (name, NULL))
           return GPG_ERR_EINVAL;
         if (getenv (name))
+#if defined(_CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES)
           {
+            int e = _putenv_s (name, "");
+
+            if (e)
+              {
+                _gpg_err_set_errno (e);
+                return -1;
+              }
+          }
+#else
+        /* Deleting an envvar.  Although the Microsoft specs for
+         * putenv tell us that one may use "NAME=" to unset an envvar,
+         * this seems not to be correct.  Thus we do the same as what
+         * we do in Unix (or well in GNU libc) and use just "NAME". */
+          {
+            char *buf;
             /* Ugly: Leaking memory.  */
             buf = _gpgrt_strdup (name);
             if (!buf)
@@ -152,6 +167,7 @@ _gpgrt_setenv (const char *name, const char *value, int overwrite)
             if (putenv (buf))
               return _gpg_err_code_from_syserror ();
           }
+#endif
         return 0;
       }
 
@@ -159,6 +175,17 @@ _gpgrt_setenv (const char *name, const char *value, int overwrite)
     if ((! exists || overwrite) && !SetEnvironmentVariable (name, value))
       return GPG_ERR_EINVAL; /* (Might also be ENOMEM.) */
     if (overwrite || !getenv (name))
+#if defined(_CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES)
+      {
+        int e = _putenv_s (name, value);
+
+        if (e)
+          {
+            _gpg_err_set_errno (e);
+            return -1;
+          }
+      }
+#else
       {
         /* Ugly: Leaking memory.  */
         buf = _gpgrt_strconcat (name, "=", value, NULL);
@@ -167,6 +194,7 @@ _gpgrt_setenv (const char *name, const char *value, int overwrite)
         if (putenv (buf))
           return _gpg_err_code_from_syserror ();
       }
+#endif
     return 0;
   }
 
@@ -228,22 +256,36 @@ _gpgrt_setenv (const char *name, const char *value, int overwrite)
 /* Convert an UTF-8 encode file name to wchar.  If the file name is
  * close to the limit of MAXPATH the API functions will fail.  The
  * method to overcome this API limitation is to use a prefix which
- * bypasses the checking by CreateFile.  This also required to first
- * convert the name to an absolute file name.  */
+ * bypasses the checking by CreateFile.  This also requires to first
+ * convert the name to an absolute file name.  To avoid looking up the
+ * fill name in all cases we do the \\?\ prefixing only if FNAME is
+ * longer than 60 byes or if it has any path separator.  */
 wchar_t *
 _gpgrt_fname_to_wchar (const char *fname)
 {
+  static int no_extlenpath;
   wchar_t *wname;
   wchar_t *wfullpath = NULL;
   int success = 0;
 
+  if (!no_extlenpath)
+    {
+      const char *s = getenv ("GPGRT_DISABLE_EXTLENPATH");
+      if (s && atoi (s))
+        no_extlenpath = 1;
+      else
+        no_extlenpath = -1;
+    }
+
   wname = _gpgrt_utf8_to_wchar (fname);
   if (!wname)
     return NULL;
 
-  if (!strncmp (fname, "\\\\?\\", 4))
+  if (no_extlenpath > 0)
+    success = 1; /* Extended length path support has been disabled.  */
+  else if (!strncmp (fname, "\\\\?\\", 4) || !strncmp (fname, "//?/", 4))
     success = 1; /* Already translated.  */
-  else if (wcslen (wname) > 230)
+  else if (strpbrk (fname, "/\\") || wcslen (wname) > 60)
     {
       int wlen = 1024;
       int extralen;
@@ -255,10 +297,16 @@ _gpgrt_fname_to_wchar (const char *fname)
       if (!wfullpath)
         goto leave;
 
-      if (*fname == '\\' && fname[1] == '\\' && fname[2])
+      if (((*fname == '\\' && fname[1] == '\\')
+           ||(*fname == '/' && fname[1] == '/')) && fname[2])
         {
-          wcscpy (wfullpath, L"\\\\?\\UNC\\");
-          extralen = 8;
+          /* Note that the GFPN call below will append the UNC name
+           * and thus two leading backslashes.  However, for an
+           * extended length path only one backslash is expected after
+           * the prefix.  We handle this by replacing the first
+           * backslash with the C from UNC after the GFPN call.  */
+          wcscpy (wfullpath, L"\\\\?\\UN");
+          extralen = 6;
         }
       else
         {
@@ -289,6 +337,10 @@ _gpgrt_fname_to_wchar (const char *fname)
       _gpgrt_free_wchar (wname);
       wname = wfullpath;
       wfullpath = NULL;
+
+      if (extralen == 6)
+        wname[6] = L'C';  /* Replace first backslash - see above.  */
+
       /* Need to make sure that all slashes are mapped. */
       for (w = wname; *w; w++)
         if (*w == L'/')
@@ -436,11 +488,7 @@ _gpgrt_chdir (const char *name)
 char *
 _gpgrt_getcwd (void)
 {
-#ifdef HAVE_W32CE_SYSTEM
-
-  return xtrystrdup ("/");
-
-#elif defined(HAVE_W32_SYSTEM)
+#if defined(HAVE_W32_SYSTEM)
   wchar_t wbuffer[MAX_PATH + sizeof(wchar_t)];
   DWORD wlen;
   char *buf, *p;
